import { PlusOutlined } from "@ant-design/icons";
import { Button, Checkbox, Form, Grid, Popconfirm, Space } from "antd";
import MyModal, {
  MyModalCloseCreateButtonFooter,
  MyModalCloseSaveButtonFooter,
} from "../../../Components/MyModal";
import { useEffect, useState } from "react";
import { myFetch, EncodeStringToBase64 } from "../../../utils";
import { useTranslation } from "react-i18next";
import MyTable from "../../../Components/MyTable";
import {
  MyAccountNameFormInput,
  MyPasswordFormInput,
  MyUsernameFormInput,
} from "../../../Components/MyFormInputs";
import { Link, useParams } from "react-router-dom";

const { useBreakpoint } = Grid;

export default function StoreEmployees() {
  const { t } = useTranslation();
  const screenBreakpoint = useBreakpoint();
  const { storeId } = useParams();

  const [employees, setEmployees] = useState([]);
  const [isRequesting, setIsRequesting] = useState(false);

  const [addEditEmployeeModalOptions, setAddEditEmployeeModalOptions] =
    useState({
      mode: "add",
      isOpen: false,
      selectedEmployee: null,
    });

  const getTableColumns = () => {
    return [
      {
        title: t("common.accountName"),
        dataIndex: "account_name",
        key: "account_name",
      },
      {
        title: t("common.username"),
        dataIndex: "username",
        key: "username",
      },
      {
        title: t("common.action"),
        dataIndex: "action",
        key: "actions",
        render: (_, record) => (
          <Space>
            <Link
              to="#"
              onClick={() => {
                setAddEditEmployeeModalOptions({
                  ...addEditEmployeeModalOptions,
                  mode: "edit",
                  isOpen: true,
                  selectedEmployee: record,
                });
              }}
            >
              {t("common.button.edit")}
            </Link>
            <Popconfirm
              title={t("employees.popConfirmDeleteEmployee.title")}
              description={t("employees.popConfirmDeleteEmployee.description")}
              okText={t("common.button.delete")}
              onConfirm={() => {
                setIsRequesting(true);

                myFetch("/users", "DELETE", {
                  userId: record.key,
                })
                  .then(() => fetchEmployees())
                  .catch((errStatus) => {
                    console.log(errStatus);

                    setIsRequesting(false);
                  });
              }}
            >
              <Link to="#">{t("common.button.delete")}</Link>
            </Popconfirm>
          </Space>
        ),
      },
    ];
  };

  const getTableItems = () => {
    return employees.map((employee) => {
      return {
        key: employee.user_id,
        account_name: employee.account_name,
        username: employee.username,
      };
    });
  };

  const fetchEmployees = () => {
    setIsRequesting(true);

    myFetch(`/users/${storeId}`, "GET")
      .then((data) => {
        setIsRequesting(false);
        setEmployees(data.employees);
      })
      .catch((errStatus) => {
        console.log(errStatus);
      });
  };

  useEffect(() => fetchEmployees(), []);

  return (
    <>
      <div
        style={{
          display: "flex",
          justifyContent: "space-between",
          flexDirection: screenBreakpoint.xs ? "column" : "row",
        }}
      >
        <h1>
          {t("employees.pageTitle")}
          {employees.length > 0 && ` (${employees.length})`}
        </h1>

        <ModalAddEditEmployee
          modalOptions={addEditEmployeeModalOptions}
          setModalOptions={setAddEditEmployeeModalOptions}
          fetchEmployees={fetchEmployees}
        />
      </div>

      <MyTable
        props={{
          loading: isRequesting,
          columns: getTableColumns(),
          dataSource: getTableItems(),
        }}
      />
    </>
  );
}

function ModalAddEditEmployee({
  modalOptions,
  setModalOptions,
  fetchEmployees,
}) {
  const { t } = useTranslation();
  const { storeId } = useParams();
  const screenBreakpoint = useBreakpoint();
  const [form] = Form.useForm();

  const [isRequesting, setIsRequesting] = useState(false);

  const handleModalClose = () => {
    setModalOptions({
      ...modalOptions,
      isOpen: false,
    });

    form.resetFields();
  };

  useEffect(() => {
    if (!modalOptions.isOpen) return;

    if (modalOptions.mode === "edit") {
      form.setFieldsValue({
        username: modalOptions.selectedEmployee.username,
        accountName: modalOptions.selectedEmployee.account_name,
      });
    }
  }, [modalOptions.isOpen]);

  return (
    <>
      <Button
        type="primary"
        block={screenBreakpoint.xs}
        icon={<PlusOutlined />}
        onClick={() =>
          setModalOptions({ ...modalOptions, mode: "add", isOpen: true })
        }
      >
        {t("employees.addEmployee")}
      </Button>

      <MyModal
        title={
          modalOptions.mode === "add"
            ? t("employees.addEmployee")
            : t("employees.editEmployee")
        }
        isOpen={modalOptions.isOpen}
        onCancel={handleModalClose}
        footer={
          modalOptions.mode === "add" ? (
            <MyModalCloseCreateButtonFooter
              onCancel={handleModalClose}
              isCreateButtonLoading={isRequesting}
              onCreate={() => {
                form
                  .validateFields()
                  .then((values) => {
                    setIsRequesting(true);

                    myFetch("/users", "POST", {
                      storeId: storeId,
                      username: values.username,
                      accountName: values.accountName,
                      password: EncodeStringToBase64(values.password),
                    })
                      .then(() => {
                        setIsRequesting(false);
                        handleModalClose();

                        fetchEmployees();
                      })
                      .catch((errStatus) => {
                        console.log(errStatus);

                        setIsRequesting(false);
                      });
                  })
                  .catch((info) => {
                    console.log("Validate Failed:", info);
                  });
              }}
            />
          ) : (
            <MyModalCloseSaveButtonFooter
              onCancel={handleModalClose}
              isSaveButtonLoading={isRequesting}
              onSave={() => {
                // only validate if something has changed
                let formUsername = form.getFieldValue("username");
                let formAccountName = form.getFieldValue("accountName");

                if (
                  formUsername === modalOptions.selectedEmployee.username &&
                  formAccountName === modalOptions.selectedEmployee.account_name
                ) {
                  handleModalClose();
                  return;
                }

                let validateFields = [];
                let body = {
                  userId: modalOptions.selectedEmployee.key,
                };

                if (formUsername !== modalOptions.selectedEmployee.username) {
                  validateFields.push("username");
                  body.username = formUsername;
                }

                if (
                  formAccountName !== modalOptions.selectedEmployee.account_name
                ) {
                  validateFields.push("accountName");
                  body.accountName = formAccountName;
                }

                form
                  .validateFields(validateFields)
                  .then(() => {
                    setIsRequesting(true);

                    myFetch("/users/update", "POST", body)
                      .then(() => {
                        setIsRequesting(false);
                        handleModalClose();

                        fetchEmployees();
                      })
                      .catch((errStatus) => {
                        console.log(errStatus);

                        setIsRequesting(false);
                      });
                  })
                  .catch((info) => {
                    console.log("Validate Failed:", info);
                  });
              }}
            />
          )
        }
      >
        <Form form={form} layout="vertical" requiredMark={false}>
          <MyUsernameFormInput />

          <MyAccountNameFormInput hasFeedback />

          {modalOptions.mode === "add" && (
            <>
              <MyPasswordFormInput />
              <Form.Item>
                <Checkbox defaultChecked disabled>
                  {t("employees.modalAddEmployee.checkboxPasswordChange")}
                </Checkbox>
              </Form.Item>
            </>
          )}
        </Form>
      </MyModal>
    </>
  );
}
